#
# stream-fishing.R,  4 Aug 24
#
# Data from:
# https://www.cs.virginia.edu/stream/#ListofTables
# Memory Bandwidth and Machine Balance in Current High Performance Computers
# John D. McCalpin

library("colorspace")
library("plyr")


par(bty="l")
par(las=1)
par(pch="+")
pal_col=rainbow(4)

st=read.csv("../stream-2009-04-03.csv")

st[st == 0]=NA  # handle the zero values

st$Date=as.Date(st$Date, format="%d/%m/%y")

plot(st$Copy/st$Add, log="y")

table(st$B.W)
# 
#   4   8 
#  43 975 
table(st$CPUs)
# 
#    1    2    3    4    5    6    7    8   10   12   14   15   16   18   20   22 
#  474   99    8   97    3   11    3   86    2    8    2    1   68    1    4    1 
#   24   28   30   32   40   48   56   63   64   72   96  120  128  144  256  512 
#   15    3    1   51    2    6    3    2   28    2    2    1   12    1   10    6 
#  768 1024 2048 
#    1    1    3 
table(st$standard.tuned.experimental)
# 
#   D   E   S   T 
#   1  47 905  65 
table(st$uniprocessor)
# 
#   N   Y 
# 711 307 
table(st$distributed.memory)
# 
#   N   Y 
# 968  50 
table(st$FLOPS.Hz)
# 
# 0.04  0.1  0.2  0.4    1  1.5    2    3    4    5    8   12   16   32  120 
#    1    1    3   16  209    7  491    5  248    7    1    1   21    6    1 
table(st$PC.Mac.Other.Cluster)
# 
#   C   M   N   P 
#   5  39 785 189 
table(st$other)
# 
#    N    Y 
# 1007   11 
range(st$date)
# [1] "1991-09-30" "2009-04-01"
nrow(st)
# [1] 1018

# Fit regression models

start_date=as.Date("1990-1-1")
end_date=as.Date("2009-1-1")
plot(st$Date, st$Copy, log="y", col=pal_col[2],
	xaxs="i",
	xlim=c(start_date, end_date),
	xlab="Date", ylab="Megabytes per second")

# c_mod=glm(log(Copy) ~ Date*Mhz+B.W, data=st)
# c_mod=glm(log(Copy) ~ Date*Mhz+B.W+PC.Mac.Other.Cluster+Shared.memory+CPUs+uniprocessor, data=st)
pc_mod=glm(log(Copy) ~ Date*Mhz+B.W+PC.Mac.Other.Cluster+CPUs, data=st)
summary(pc_mod)

c_mod=glm(log(Copy) ~ Date*Mhz+Shared.memory+CPUs+uniprocessor, data=st)
summary(c_mod)

d_range=seq(start_date, end_date, by=92) # approx 92 days in a quarter

Mhz_mod=glm(log(Mhz) ~ Date, data=st)
summary(Mhz_mod)

Mhz_pred=predict(Mhz_mod, newdata=data.frame(Date=d_range))

pred=predict(pc_mod, newdata=data.frame(Date=d_range, Mhz=exp(Mhz_pred), B.W=8, PC.Mac.Other.Cluster="P", CPUs=1))
lines(d_range, exp(pred), col=pal_col[1])

pred=predict(pc_mod, newdata=data.frame(Date=d_range, Mhz=exp(Mhz_pred), B.W=8, PC.Mac.Other.Cluster="M", CPUs=1))
lines(d_range, exp(pred), col=pal_col[3])

pred=predict(pc_mod, newdata=data.frame(Date=d_range, Mhz=exp(Mhz_pred), B.W=8, PC.Mac.Other.Cluster="N", CPUs=1))
lines(d_range, exp(pred), col=pal_col[4])

legend("bottomright", legend=c("Expensive", "PC", "Mac"), cex=1.2,
                        fill=pal_col[c(4, 1, 3)], border="white", bty="n")

dev.copy(dev=png, file="MBsec-date.png")
dev.off()


